require 'time'
require 'date'
require 'digest/sha1'

require 'validating_base'

class ViewTerm < ValidatingBase
  def initialize(hash)
    @start_year = hash[:start_year].to_i
    @start_mon = hash[:start_mon].to_i
    @end_year = hash[:end_year].to_i
    @end_mon = hash[:end_mon].to_i
    @ctr = hash[:ctr].to_i

    if Date.exist?(@start_year, @start_mon, 1)
      @sd = Date.new(@start_year, @start_mon, 1)
    end
    if Date.exist?(@end_year, @end_mon, 1)
      @ed = (Date.new(@end_year, @end_mon, 1) >> 1) - 1
    end
  end
  attr_reader :start_year, :start_mon, :end_year, :end_mon
  attr_reader :sd, :ed, :ctr
  
  def validate
    errors.add_to_base('開始月と終了月の関係が正しくありません。') if @sd > @ed
  end
end

# 総勘定元帳
class GeneralLedgerController < ApplicationController
  before_filter :login_required
  
  def index
  end
  
  # 部門の予算・実績対比表の出力
  def comparison
    @term = ViewTerm.new params[:term]
    @term.validate
    @kubuns = Kubun.find params[:kubun]
    if !@term.errors.empty?
      render :action => 'index'
      return
    end

    @cdiv = current_user.division.default_controller
    raise if !@cdiv
    
    @gl = GeneralLedger.find :all,
              :select => 'date, kubun_id, division_id, account_title_id, sum(amount) as amount',
              :conditions => [<<EOF, @term.sd, @term.ed, @cdiv.id],
(kubun_id IN (#{params[:kubun].join(',')})) AND
(date BETWEEN ? AND ?) AND
division_id IN (SELECT division_id FROM controllers_divisions
                                   WHERE control_division_id = ?)
EOF
              :group => 'kubun_id, division_id, account_title_id, date'
    
  end
  
  def view
  end
  
  def view2
    @term = ViewTerm.new params[:term]
    @term.validate
    @kubuns = Kubun.find params[:kubun]
    @controllers = ControlDivision.find :all, :conditions => ['parent_id = ?', @term.ctr]
    
    @vs = Digest::SHA1.hexdigest Time.now.to_f.to_s
    session[@vs] = {
      :term => @term,
      :kubuns => params[:kubun],
      :root => @term.ctr }
  end
  
  # 実績データの削除
  def clear
    term = ViewTerm.new(params[:term])
    raise if !term.sd || !term.ed || term.sd > term.ed
    
    actual_kubun = Kubun.find_by_name('actual')
    GeneralLedger.delete_all ['kubun_id = ? and date between ? and ?',
                              actual_kubun.id, term.sd, term.ed]
  end
  
  # 科目別元帳
  def item_ledger
    term = ViewTerm.new(params[:term])
    raise if !term.sd || !term.ed || term.sd > term.ed
    
    actual_kubun = Kubun.find_by_name('actual')
    item = AccountTitle.find(params[:item][:id])
    @gl = GeneralLedger.find(:all,
          :conditions => ['kubun_id = ? and (date between ? and ?) and account_title_id = ?',
             actual_kubun.id, term.sd, term.ed, item.id],
          :include => :account_title)

    # 出力
    headers["Content-Type"] = "application/vnd.ms-excel; charset=UTF-8"
    headers["Content-Disposition"] = "attachment;\n  filename*=iso-2022-jp'ja'" +
                                     NKF.nkf("-m0 -j -W", "科目別元帳.xls")
    render :action => 'item_ledger', :layout => false
  end
  
  # 予算・実績対比表のダウンロード
  def download
    vs_key = params[:vs]
    term = session[vs_key][:term]
    root = ControlDivision.find session[vs_key][:root]
    kubuns = session[vs_key][:kubuns]
    control_division = ControlDivision.find(params[:division]) if params[:division]
    
    if control_division
      cond = [<<EOF, term.sd, term.ed, control_division.id]
(kubun_id IN (#{kubuns.join(',')})) AND
(date BETWEEN ? AND ?) AND
division_id IN (SELECT division_id FROM controllers_divisions
                                   WHERE control_division_id = ?)
EOF
    else
      cond = [<<EOF, term.sd, term.ed]
(kubun_id IN (#{kubuns.join(',')})) AND (date BETWEEN ? AND ?)
EOF
    end

    gl = GeneralLedger.find :all,
                            :select => "general_ledger.*",
                            :conditions => cond

    data = <<EOF
<html xmlns:o="urn:schemas-microsoft-com:office:office"
      xmlns:x="urn:schemas-microsoft-com:office:excel"
      xmlns="http://www.w3.org/1999/xhtml" xml:lang="ja" lang="ja"<
<head>
  <meta http-equiv='Content-Type' content='text/html;charset=utf-8'>
  <meta name='ProgId' content='Excel.Sheet'>
  <xml>
    <o:DocumentProperties>
      <o:Created>#{Time.now.iso8601}</o:Created>
      <o:Company>●</o:Company>
    </o:DocumentProperties>
  </xml>
  <style>
    @page {
        margin:.59in .39in .39in .39in;
        mso-header-margin:.51in;
        mso-footer-margin:.31in;
        mso-page-orientation:landscape;}
    td {
      font-size:9.0pt; }
  </style>
</head>
<body>
    <h1>#{control_division ? control_division.name : nil} #{term.sd}-#{term.ed} 実績</h1>
<table border="0" cellpadding="0" cellspacing="0" width="100%"
    style='border-collapse:collapse;table-layout:fixed;'>
<tr>
  <th>伝票日付</th>
  <th>年月</th>
  <th>区分</th>
  <th>伝票NO</th>
  <th>作成日</th>
  <th>作成者コード</th>
  <th>作成者名</th>
  <th>科目コード</th>
  <th>補助コード</th>
  <th>科目名</th>
  <th>科目名2</th>
  <th>部門コード</th>
  <th>部門名</th>
  <th>金額</th>
  <th>摘要</th>
</tr>
EOF
    not_output = [722, 801, 802, 803, 804, 805, 806, 807, 808, 809, 810, 831, 832, 833, 834, 835, 836, 837, 838, 905, 913, 915]
    # TODO: 決めうちしてる。fixme
    
    gl.each {|r|
      # 科目の抽出
      next if not_output.include?(r.account_title.account_code) ||
                r.account_title.account_code < 601

      data << <<EOF
<tr>
  <td>#{r.date}</td>
  <td>#{sprintf("%04d%02d", r.date.year, r.date.mon)}</td>
  <td>#{Kubun.find(r.kubun_id).long_name}</td>
  <td>#{r.has_attribute?('slip_number') ? r.slip_number : ''}</td>
  <td>#{r.has_attribute?('created_at') && r.created_at ? r.created_at.strftime("%Y/%m/%d") : ''}</td>
  <td>#{r.has_attribute?('created_by') && r.created_by ? r.created_by.worker_code : nil}</td>
  <td>#{r.has_attribute?('created_by') && r.created_by ? r.created_by.name : nil}</td>
  <td>#{r.account_title.account_code}</td>
  <td>#{r.account_title.suppl_code}</td>
  <td>#{r.account_title.name}</td>
  <td>#{r.account_title.account_code}#{r.account_title.main_name}
  <td>#{control_division ? r.division.division_code : nil}</td>
  <td>#{control_division ? r.division.name :
            (r.division ? r.division.controller(root).name : '')}</td>
  <td>#{r.amount}</td>
  <td>#{r.has_attribute?('remarks') ? r.remarks : ''}</td>
</tr>
EOF
    }
    data << "</table></body></html>"
    
    send_data data, :type => "application/vnd.ms-excel; charset=UTF-8", :disposition => %Q(attachment; filename="#{
    sprintf("%04d%02d", term.ed.year, term.ed.mon)}#{control_division ? control_division.name : 'all'}.xls")
  end
end

